@@ -21,6 +21,10 @@ plapply <- function(workList,worker,parallelCluster) {
     return(rowlist[[1]])
   }
   # see if a library can supply a fast method
+  if(requireNamespace("data.table", quietly = TRUE)) {
+    return(as.data.frame(data.table::rbindlist(rowlist),
+                         stringsAsFactor=FALSE))
+  }
   if(requireNamespace("dplyr", quietly = TRUE)) {
     return(as.data.frame(dplyr::bind_rows(rowlist),
                          stringsAsFactor=FALSE))
@@ -62,7 +66,7 @@ plapply <- function(workList,worker,parallelCluster) {
     # factor, character case
     return(as.character(xcol))
   }
-  if(is.logical(xcol)||is.numeric(xcol)) { # is.numeric(factor('a')) returns false, but lets not have factors here anyway
+  if(is.logical(xcol)||is.numeric(xcol)||is.integer(xcol)) { # is.numeric(factor('a')) returns false, but lets not have factors here anyway
     # logical (treat as an indicator), integer, numeric case
     # is.numeric(1:10) returns TRUE (integers get into this block)
     return(as.numeric(xcol))
@@ -216,20 +220,35 @@ linScore <- function(varName,xcol,ycol,weights,numberOfHiddenDegrees=0) {
 #' 
 catScore <- function(varName,x,yC,yTarget,weights,numberOfHiddenDegrees=0) {
   if(is.null(weights)) {
-    weights <- 1.0+numeric(length(x))
+    weights <- rep(1.0, length(x))
   }
   pRsq <- 0.0
   sig <- 1.0
   a <- 0.0
   if(.has.range.cn(x) && 
      .has.range.cn(as.numeric(yC==yTarget))) {
-    tf <- data.frame(x=x,y=(yC==yTarget),
+    tfp <- data.frame(x = x,
+                     y = (yC==yTarget),
+                     w = weights,
                      stringsAsFactors=FALSE)
+
     suppressWarnings(tryCatch({      
       model <- stats::glm(stats::as.formula('y~x'),
-                          data=tf,
+                          data=tfp,
                           family=stats::binomial(link='logit'),
-                          weights=weights)
+                          weights=tfp$w)
+      if(!model$converged) { 
+        # put in small oposite to prevent unbounded models
+        # try to fix non-converge
+        tfo <- tfp
+        tfo$w <- tfo$w*1.0e-5
+        tfo$y <- !tfo$y
+        tf <- rbind(tfp, tfo)
+        model <- stats::glm(stats::as.formula('y~x'),
+                            data=tf,
+                            family=stats::binomial(link='logit'),
+                            weights=tf$w)
+      }
       if(model$converged) {
         delta_deviance <- model$null.deviance - model$deviance
         if((model$null.deviance>0)&&(delta_deviance>0)) {
